/* Program  hr.cc                    */
/* By Harald Stauss                  */
/* Last change 11 Oct. 2000          */

#include "tsa.h"
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>

/* Global variables */
REGIST   r;

/* function find_dia */
void find_dia(short ch, unsigned long n1, unsigned long n2, float schwelle,
	      unsigned long *dia, unsigned long *sys) {
  unsigned long n;
  n=n1+1;
  do {
    /* suche Diastole */
    while((r.ch[ch].data[n]>=r.ch[ch].data[n-1] ||
	   r.ch[ch].data[n]>r.ch[ch].data[n+1]) && (n<n2))
      n++;
    *dia=n;
    /* suche Systole */
    while((r.ch[ch].data[n]<r.ch[ch].data[n-1] ||
	   r.ch[ch].data[n]<=r.ch[ch].data[n+1]) && (n<n2))
      n++;
    *sys=n;
  }
  while ((r.ch[ch].data[*sys]-r.ch[ch].data[*dia])<schwelle && n<n2);
  if (n>=n2) {*dia=n2; *sys=n2;};
  return;
}

main(int argc, char *argv[])
{
  short         i;
  char          *fin, *ftin, *finbuf, *ftinbuf, *fout, *foutbuf;
  short         bpch=0;
  float         thres=20, sr;
  unsigned long na, ne, n, ni, nampli, lampli=1000;
  unsigned long ndia, nsys, ndia_alt, nsys_alt;
  float         *time, *hr, *sys, *mea, *dia;
  ofstream      datei;


  finbuf  = new char[255];
  fin     = finbuf;
  ftinbuf = new char[255];
  ftin    = ftinbuf;
  foutbuf = new char[255];
  fout    = foutbuf;

  time    = new float[lampli];
  hr      = new float[lampli];
  sys     = new float[lampli];
  mea     = new float[lampli];
  dia     = new float[lampli];


  printf("HR Ver.1.1 by H. Stauss, 2000 \n");

  if ((argc != 5) && (argc !=6))
    {
      printf("Number of parameters wrong! \n");
      printf("Computes HR and MAP from direct BP signal \n");
      printf("HR fin fout ch thres [SR] \n");
      printf("fin    : name of input file (*.TSA or *.ASC). \n");
      printf("fout   : name of output file (will be a text file). \n");
      printf("ch     : BP - channel (starts with 0). \n");
      printf("thres  : threshold (try 20). \n");
      printf("SR     : sampling rate in case of *.ASC. \n");
      goto ende;
    }

  strcpy(fin,argv[1]);
  strcpy(fout,argv[2]);
  strcpy(ftin,fin);
  ftin=strrchr(ftin,'.');
  for (i=0; ftin[i]; i++) ftin[i]=toupper(ftin[i]);

  sscanf(argv[3],"%d",&bpch);
  sscanf(argv[4],"%f",&thres);
  if (argc==6) sscanf(argv[5],"%f",&sr);
  if ((strcmp(ftin,".ASC")==0) && (argc!=6)) {
    cout << "Need sampling rate for ASCII Files!\n";
    goto ende;
  }

  printf("Input file           : %s \n",fin);
  printf("Output file          : %s \n",fout);

  /* E i n g a b e d a t e i   o e f f n e n */
  if (strcmp(ftin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(ftin,".ASC")==0) r.ReadASCII(fin,sr);
  if ((strcmp(ftin,".TSA")!=0) && (strcmp(ftin,".ASC")!=0)) {
    cout << "File type for input not recognized! \n";
    goto ende;
  }

  if (bpch>=0 && bpch<r.n) {
    na=0; ne=r.ch[bpch].n;
    n=na; nampli=0;
    find_dia(bpch, n, ne, thres, &ndia, &nsys);
    ndia_alt=ndia; nsys_alt=nsys; n=nsys;
    while (n<ne) {
      find_dia(bpch, n, ne, thres, &ndia, &nsys);
      if(ndia<ne){
	if (nampli>=lampli) {
	  lampli+=1000;
	  hr=(float *)realloc(hr,lampli*sizeof(float));
	  sys=(float *)realloc(sys,lampli*sizeof(float));
	  mea=(float *)realloc(mea,lampli*sizeof(float));
	  dia=(float *)realloc(dia,lampli*sizeof(float));
	  time=(float *)realloc(time,lampli*sizeof(float));
	}
	hr[nampli]=60*r.ch[bpch].sr/(ndia-ndia_alt);
	time[nampli]=nsys_alt/r.ch[bpch].sr;
	sys[nampli]=r.ch[bpch].data[nsys_alt];
	mea[nampli]=0;
	for (ni=nsys_alt; ni<nsys; ni++) mea[nampli]+=r.ch[bpch].data[ni];
	mea[nampli]=mea[nampli]/(nsys-nsys_alt);
	dia[nampli]=r.ch[bpch].data[ndia_alt];
	nampli++;
      }
      ndia_alt=ndia; nsys_alt=nsys;
      n=nsys;
    }
    
    // A u s g a b e d a t e i   s c h r e i b e n ..........;
    datei.open(fout);
    for (n=1; n<nampli-1; n++) { /* first and last are sometimes incorrect */
      datei << time[n] << '\t' << sys[n] <<'\t' << mea[n] << '\t' << dia[n] << '\t' << hr[n] << '\n';
    }
    datei.close();

    cout << "Average sampling rate: " << nampli/time[nampli-1] << "Hz \n";
  }

 ende:

  delete []time; delete[]hr; delete[]sys; delete[]mea; delete[]dia;
  delete []finbuf;
  delete []ftinbuf;
  delete []fout;

  return(0);
} /* Ende von main */
